defmodule ClickHouse.Config do
  @moduledoc false

  def new(client, opts) do
    table = table_name(client)
    :ets.new(table, [:named_table, read_concurrency: true])

    init(table, :interface, opts)
    init(table, :formats, opts)
  end

  def interface(client), do: fetch(client, :interface)

  def format(client, name), do: fetch(client, {:format, name})

  defp fetch(client, key) do
    case :ets.lookup(table_name(client), key) do
      [{^key, value}] -> {:ok, value}
      _ -> {:error, :not_found}
    end
  rescue
    ArgumentError -> {:error, :not_found}
  end

  defp table_name(client) do
    :"#{__MODULE__}.#{client}"
  end

  defp init(table, :interface, opts) do
    interface = Keyword.fetch!(opts, :interface)
    :ets.insert(table, {:interface, interface})
  end

  defp init(table, :formats, opts) do
    formats = Keyword.fetch!(opts, :formats)

    for {module, names} <- formats do
      for name <- names do
        :ets.insert(table, {{:format, name}, module})
      end
    end
  end
end
